//    Copyright (C) 2003 Chad Brassil
//    This file is part of NFP.
//
//    NFP is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    Foobar is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with Foobar; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include <PalmOS.h>
#include <SysEvtMgr.h>

#include "NFP.h"
#include "NFPRsc.h"

#define RECORD_DB_TYPE  ('Data')
// Max len of sound trigger label placeholder
#define soundTriggerLabelLen			26 //needs to be no longer than length of SetAlarmsMorningPopTrigger and SetAlarmsMorningPopTrigger
//#define defaultAlarmSoundNameID			1800	// must be equal to alarm name in General Panel's MidiSounds.rsrc


// handle to the list containing names and DB info of MIDI tracks.
// Each entry is of type SndMidiListItemType.
static MemHandle	gMidiListH;
// number of entries in the MIDI list
static UInt16	gMidiCount;

// Placeholder for sound trigger label
static Char * soundTriggerLabelPmor;
static Char * soundTriggerLabelPeve;




// FUNCTION:    PlayAlarmSound
//
// DESCRIPTION:	Play a MIDI sound given a unique record ID of the MIDI record in the System
//						MIDI database.  If the sound is not found, then the default alarm sound will
//						be played.
//
// PARAMETERS:  uniqueRecID	-- unique record ID of the MIDI record in the System
//										   MIDI database.
//
// RETURNED:    nothing
//
// REVISION HISTORY:
//			Name	Date		Description
//			----	----		-----------
//			vmk	8/10/97	Initial version
//			trev	08/14/97	Ported to dateBook App
//			vmk	12/11/97	Prevent call to DmFindRecordByID if DmOpenDatabase failed

static void SECT1 PlayAlarmSound(UInt32 uniqueRecID)
{
	Err			err;
	MemHandle	midiH;							// handle of MIDI record
	SndMidiRecHdrType*	midiHdrP;			// pointer to MIDI record header
	UInt8*		midiStreamP;					// pointer to MIDI stream beginning with the 'MThd'
														// SMF header chunk
	UInt16		cardNo;							// card number of System MIDI database
	LocalID		dbID;								// Local ID of System MIDI database
	DmOpenRef	dbP = NULL;						// reference to open database
	UInt16		recIndex;						// record index of the MIDI record to play
	SndSmfOptionsType	smfOpt;					// SMF play options
	DmSearchStateType	searchState;			// search state for finding the System MIDI database
	Boolean		bError = false;				// set to true if we couldn't find the MIDI record
	
		
	// Find the system MIDI database
	err = DmGetNextDatabaseByTypeCreator(true, &searchState,
			 		sysFileTMidi, sysFileCSystem, true, 
			 		&cardNo, &dbID);
	if ( err )
		bError = true;														// DB not found
	
	// Open the MIDI database in read-only mode
	if ( !bError )
		{
		dbP = DmOpenDatabase (cardNo, dbID, dmModeReadOnly);
		if ( !dbP )
			bError = true;													// couldn't open
		}
	
	// Find the MIDI track record
	if ( !bError )
		{
		err = DmFindRecordByID (dbP, uniqueRecID, &recIndex);
		if ( err )
			bError = true;														// record not found
		}
		
	// Lock the record and play the sound
	if ( !bError )
		{
		// Find the record handle and lock the record
		midiH = DmQueryRecord(dbP, recIndex);
		midiHdrP = (SndMidiRecHdrType*)MemHandleLock(midiH);
		
		// Get a pointer to the SMF stream
		midiStreamP = (UInt8*)midiHdrP + midiHdrP->bDataOffset;
		
		// Play the sound (ignore the error code)
		// The sound can be interrupted by a key/digitizer event
		smfOpt.dwStartMilliSec = 0;
		smfOpt.dwEndMilliSec = sndSmfPlayAllMilliSec;
		smfOpt.amplitude = PrefGetPreference(prefAlarmSoundVolume);
		smfOpt.interruptible = true;
		smfOpt.reserved = 0;
		err = SndPlaySmf (NULL, sndSmfCmdPlay, midiStreamP, &smfOpt, NULL, NULL, false);
		
		// Unlock the record
		MemPtrUnlock (midiHdrP);
		}
	
	// Close the MIDI database
	if ( dbP )
		DmCloseDatabase (dbP);
		
	
	// If there was an error, play the alarm sound resource stored with the app.
	// This is equivalent to SndPlaySystemSound(sndAlarm), but the sound is
	// interruptible.
	if ( bError )
	{
		//the following function is only supported in 3.2 and higher
		//SndPlaySmfResource(midiRsc, alarmSoundID, prefAlarmSoundVolume);
		//therefore I subbed in the next line, although not interruptible
		SndPlaySystemSound(sndAlarm);
	}
}


// FUNCTION:    SetSoundLabel
//
// DESCRIPTION: Sets the sound trigger label, using truncation
//
// PARAMETERS:  formP - the form ptr
//              labelP - ptr to original label text
//
// RETURNED:    nothing
//
// REVISION HISTORY:
//			Name		Date		Description
//			----		----		-----------
//			frigino	8/18/97	Initial Revision
//			Chad			Revised to use standard API calls


static void SECT2 SetSoundLabelmor(FormPtr formP, const char* labelP)
{
//	ControlPtr	triggerP;
	ControlType *ctlP;
//	UInt16		triggerIdx;
//	Char*	pLabelString;
//	UInt16		labelLen;
	
	
	GetObjectPtr(ctlP,SetAlarmsMorningSoundPopTrigger);
	//dynamically create label to save space
//	pLabelString = (Char*)CtlGetLabel(ctlP); //this may cause problems
//	labelLen = soundTriggerLabelLen; //StrLen(pLabelString); //otherwise continually shrinks
	StrNCopy(soundTriggerLabelPmor,labelP,soundTriggerLabelLen-1);
	// Terminate string at max len
	soundTriggerLabelPmor[soundTriggerLabelLen - 1]=chrNull;
	CtlSetLabel(ctlP, soundTriggerLabelPmor);

	// Copy the label, winUp to the max into the ptr
//	StrNCopy(soundTriggerLabelP, labelP, soundTriggerLabelLen);
	
	// Terminate string at max len
//	soundTriggerLabelP[soundTriggerLabelLen - 1] = '\0';
	// Get trigger idx
//	triggerIdx = FrmGetObjectIndex(formP, SetAlarmsMorningSoundPopTrigger);
	// Get trigger control ptr
//	triggerP = (ControlType*)FrmGetObjectPtr(formP, triggerIdx);
	// Use category routines to truncate it
//	CategoryTruncateName(soundTriggerLabelP, ResLoadConstant(soundTriggerLabelWidth));
	// Set the label
//	CtlSetLabel(triggerP, soundTriggerLabelP);
}

static void SECT2 SetSoundLabeleve(FormPtr formP, const char* labelP)
{
	ControlType *ctlP;
	
	GetObjectPtr(ctlP,SetAlarmsEveningSoundPopTrigger);
	StrNCopy(soundTriggerLabelPeve,labelP,soundTriggerLabelLen-1);
	// Terminate string at max len
	soundTriggerLabelPeve[soundTriggerLabelLen - 1]=chrNull;
	CtlSetLabel(ctlP, soundTriggerLabelPeve);
}


// FUNCTION:    MidiPickListDrawItem
//
// DESCRIPTION: Draw a midi list item.
//
// PARAMETERS:  itemNum - which shortcut to draw
//					 bounds - bounds in which to draw the text
//					 unusedP - pointer to data (not used)
//
// RETURNED:    nothing
//
// REVISION HISTORY:
//			Name	Date		Description
//			----	----		-----------
//			vnk		8/8/97	Initial version
//			trev		8/14/97	Ported to dateBook App
//			frigino	8/18/97	Modified to truncate items in list with ...


static void SECT2 MidiPickListDrawItem (Int16 itemNum, RectanglePtr bounds, Char **unusedP)
{
//CODEWARRIOR SPECIFIC:  removed to eliminate prc-tools warning
//#pragma unused (unusedP)

	Char *	itemTextP;
	Int16		itemTextLen;
	Int16		itemWidth;
	SndMidiListItemType*	listP;
	
	ErrNonFatalDisplayIf(itemNum >= gMidiCount, "index out of bounds");
	
	// Bail out if MIDI sound list is empty
	if (gMidiListH == NULL)
		return;
	
	listP = (SndMidiListItemType*)MemHandleLock(gMidiListH);

	itemTextP = listP[itemNum].name;

	// Truncate the item with an ellipsis if it doesnt fit in the list width.
	// Get the item text length
	itemTextLen = StrLen(itemTextP);
	// Get the width of the text
	itemWidth = FntCharsWidth(itemTextP, itemTextLen);
	// Does it fit?
	if (itemWidth <= bounds->extent.x)
		{
		// Draw entire item text as is
		WinDrawChars(itemTextP, itemTextLen, bounds->topLeft.x, bounds->topLeft.y);
		}
	else
		{
		// We're going to truncate the item text
		Boolean	ignored;
		char		ellipsisChar = chrEllipsis;
		// Set the new max item width
		itemWidth = bounds->extent.x - FntCharWidth(ellipsisChar);
		// Find the item length that fits in the bounds minus the ellipsis
		FntCharsInWidth(itemTextP, &itemWidth, &itemTextLen, &ignored);
		// Draw item text that fits
		WinDrawChars(itemTextP, itemTextLen, bounds->topLeft.x, bounds->topLeft.y);
		// Draw ellipsis char
		WinDrawChars(&ellipsisChar, 1, bounds->topLeft.x + itemWidth, bounds->topLeft.y);
		}

	// Unlock list items
	MemPtrUnlock(listP);
}


// FUNCTION:    CreateMidiPickList
//
// DESCRIPTION: Create a list of midi sounds available.
//
// PARAMETERS:	listP	-- the list to contain the panel list
//					funcP	-- item draw function
//
// RETURNED:    panelCount and panelIDsP are set
//
// REVISION HISTORY:
//			Name		Date		Description
//			----		----		-----------
//			vmk		8/8/97	Initial version
//			trev		8/14/97	Ported to dateBook App
//			frigino	8/20/97	Added maximum widening of MIDI sound list

static void SECT2 MakeMidiPickList(void)
{
	Boolean	bSuccess;
	
	// Load list of midi record entries
	bSuccess = SndCreateMidiList(sysFileCSystem, false, &gMidiCount, &gMidiListH);
	if ( !bSuccess )
	{
		gMidiListH = 0;
		gMidiCount = 0;
	}
	
	return;
}

static void SECT2 FitMidiPickList(FormPtr frmP,ListPtr listP,UInt16 lstID, ListDrawDataFuncPtr funcP)
{
	SndMidiListItemType*	midiListP;
	UInt16		i;
	UInt16		listWidth;
	UInt16		maxListWidth;
	UInt16 		objIndex;
	RectangleType r;
	RectangleType rList;

	if ( gMidiListH == 0 && gMidiCount == 0 )
		return;
		
	// Now set the list to hold the number of sounds found.  There
	// is no array of text to use.
	LstSetListChoices(listP, NULL, gMidiCount);
	
	// Now resize the list to the number of panel found
	LstSetHeight (listP, gMidiCount);

	// Because there is no array of text to use, we need a function
	// to interpret the panelIDsP list and draw the list items.
	LstSetDrawFunction(listP, funcP);

	// Make the list as wide as possible to display the full sound names
	// when it is popped winUp.

	// Lock MIDI sound list
	midiListP = (SndMidiListItemType*)MemHandleLock(gMidiListH);
	// Initialize max width
	maxListWidth = 0;
	// Iterate through each item and get its width
	for (i = 0; i < gMidiCount; i++)
		{
			// Get the width of this item
			listWidth = FntCharsWidth(midiListP[i].name, StrLen(midiListP[i].name));
			// If item width is greater that max, swap it
			if (listWidth > maxListWidth)
				{
				maxListWidth = listWidth;
				}
		}
	// Unlock MIDI sound list
	MemPtrUnlock(midiListP);
	
	// Set list width to max width + left margin
	objIndex = FrmGetObjectIndex(frmP,lstID);
	FrmGetObjectBounds(frmP,objIndex,&rList);
	rList.extent.x = maxListWidth + 2;

	// Get pref dialog window extent
	FrmGetFormBounds(frmP, &r);
	// Make sure width is not more than window extent
	if (rList.extent.x > r.extent.x)
		{
			rList.extent.x = r.extent.x;
		}
	// Move list left if it doesnt fit in window
	if (rList.topLeft.x + rList.extent.x > r.extent.x)
		{
			rList.topLeft.x = r.extent.x - rList.extent.x;
		}

	FrmSetObjectBounds(frmP,objIndex,&rList);
}


// FUNCTION:    FreeMidiPickList
//
// DESCRIPTION: Free the list of midi sounds available.
//
// PARAMETERS:	none
//
// RETURNED:    nothing
//
// REVISION HISTORY:
//			Name	Date		Description
//			----	----		-----------
//			vmk	8/11/97	Initial version
//			trev	08/14/97	Ported to dateBook App


static void SECT2 FreeMidiPickList(void)
{
	if ( gMidiListH )
		{
		MemHandleFree(gMidiListH);
		gMidiListH = 0;
		gMidiCount = 0;
		}
}


/***********************************************************************
 *
 * FUNCTION:    FreeAll
 *
 * DESCRIPTION: Frees all data allocated for the duration of the dialog
 *
 * PARAMETERS:  none
 *
 * RETURNED:    none
 *
 * REVISION HISTORY:
 *			Name		Date		Description
 *			----		----		-----------
 *			frigino	8/20/97	Initial Revision
 *			vmk		12/9/97	Check soundTriggerLabelP for null before deleting
 *
 ***********************************************************************/

static void SECT2 FreeAll(void)
{
	// Free the MIDI pick list
	FreeMidiPickList();

	// Free the sound trigger label placeholder
	if ( soundTriggerLabelPmor )
	{
		MemPtrFree(soundTriggerLabelPmor);
		soundTriggerLabelPmor = NULL;
	}
		
	if ( soundTriggerLabelPeve )
	{
		MemPtrFree(soundTriggerLabelPeve);
		soundTriggerLabelPeve = NULL;
	}
}


static void SECT2 SetAlarmsFormInit(FormType * frmP)
{
	ControlType *ctlP;
	ListType *lstP;
	UInt16			item;
	UInt16			i;
	SndMidiListItemType*	midiListP;
//	Char *		defaultName;

	//Fill in current record information into form
    GetObjectPtr(ctlP,SetAlarmsMorningAlarmCheckbox);
    CtlSetValue(ctlP,g_prefs.enableMorningAlarm);
    
    GetObjectPtr(ctlP,SetAlarmsEveningAlarmCheckbox);
    CtlSetValue(ctlP,g_prefs.enableEveningAlarm);
    
	GetObjectPtr(lstP,SetAlarmsMorningRepeatList);
	LstSetSelection(lstP,g_prefs.morningAlarmRepeat);
	GetObjectPtr(ctlP,SetAlarmsMorningRepeatPopTrigger);
	CtlSetLabel (ctlP,LstGetSelectionText(lstP,g_prefs.morningAlarmRepeat));
	
	GetObjectPtr(lstP,SetAlarmsEveningRepeatList);
	LstSetSelection(lstP,g_prefs.eveningAlarmRepeat);
	GetObjectPtr(ctlP,SetAlarmsEveningRepeatPopTrigger);
	CtlSetLabel (ctlP,LstGetSelectionText(lstP,g_prefs.eveningAlarmRepeat));
	
    GetObjectPtr(ctlP,SetAlarmsDisableAlarmsCheckbox);
    CtlSetValue(ctlP,g_prefs.disableAlarms);
	
	//only make once
	MakeMidiPickList();
	
	GetObjectPtr(lstP,SetAlarmsMorningSoundList);
	FitMidiPickList(frmP,lstP, SetAlarmsMorningSoundList, MidiPickListDrawItem);
	// Traverse MIDI pick list and find the item whose unique ID matches our
	// saved unique ID and use its index as our list selection index. If we
	// don't find a match, then we want to use the MIDI sound that corresponds
	// to our default sound name; if that's not found, use item 0.

	// Default to first sound in list
	item = 0;
	// Lock MIDI sound list
	if ( gMidiListH )
	{
		midiListP = (SndMidiListItemType*)MemHandleLock(gMidiListH);

		// Iterate through each item and get its unique ID
		for (i = 0; i < gMidiCount; i++)
		{
			if (midiListP[i].uniqueRecID == g_prefs.morningAlarmSound)
			{
				item = i;
				break;		// exit for loop
			}

		}
				
		// Set the list selection
		LstSetSelection (lstP, item);

		// Init the sound trigger label
		// Create a new ptr to hold the label
		soundTriggerLabelPmor = (Char*)MemPtrNew(soundTriggerLabelLen);
		// Check for mem failure
		ErrFatalDisplayIf(soundTriggerLabelPmor == NULL, "Out of memory");
		// Set the trigger label
		SetSoundLabelmor(frmP, midiListP[item].name);
		// set into preferences in case they never click on PopTrigger and pref is still default of 0 or something else
		g_prefs.morningAlarmSound = midiListP[item].uniqueRecID;

		// Unlock MIDI sound list
		MemPtrUnlock(midiListP);
	}
		
	GetObjectPtr(lstP,SetAlarmsEveningSoundList);
	FitMidiPickList(frmP,lstP, SetAlarmsEveningSoundList, MidiPickListDrawItem);
	// Default to first sound in list
	item = 0;
	// Lock MIDI sound list
	if ( gMidiListH )
	{
		midiListP = (SndMidiListItemType*)MemHandleLock(gMidiListH);

		// Iterate through each item and get its unique ID
		for (i = 0; i < gMidiCount; i++)
		{
			if (midiListP[i].uniqueRecID == g_prefs.eveningAlarmSound)
			{
				item = i;
				break;		// exit for loop
			}

		}
				
		// Set the list selection
		LstSetSelection (lstP, item);

		// Init the sound trigger label
		// Create a new ptr to hold the label
		soundTriggerLabelPeve = (Char*)MemPtrNew(soundTriggerLabelLen);
		// Check for mem failure
		ErrFatalDisplayIf(soundTriggerLabelPeve == NULL, "Out of memory");
		// Set the trigger label
		SetSoundLabeleve(frmP, midiListP[item].name);
		// set into preferences in case they never click on PopTrigger and pref is still default of 0 or something else
		g_prefs.eveningAlarmSound = midiListP[item].uniqueRecID;

		// Unlock MIDI sound list
		MemPtrUnlock(midiListP);
	}

}
	//sound stuff pulled from Datebook
/*
	FormPtr		formP;
	Char *		label;
	Char *		textP;
	MemHandle		textH;
	FieldPtr		fld;
	ControlPtr	ctl;
	ListPtr		listP;
	UInt16			item;
	UInt16			i;
	UInt16			fldIndex;
	SndMidiListItemType*	midiListP;
	Char *		defaultName;
	
		listP = FrmGetObjectPtr(formP, FrmGetObjectIndex(formP, PreferAlarmSoundList));

	CreateMidiPickList(listP, MidiPickListDrawItem);

	// Traverse MIDI pick list and find the item whose unique ID matches our
	// saved unique ID and use its index as our list selection index. If we
	// don't find a match, then we want to use the MIDI sound that corresponds
	// to our default sound name; if that's not found, use item 0.

	// Default to first sound in list
	item = 0;

	// Lock MIDI sound list
	if ( gMidiListH )
		{
		midiListP = MemHandleLock(gMidiListH);
		defaultName = MemHandleLock(DmGetResource(strRsc, defaultAlarmSoundNameID));

		// Iterate through each item and get its unique ID
		for (i = 0; i < gMidiCount; i++)
			{
			if (midiListP[i].uniqueRecID == PrefSoundUniqueRecID)
				{
				item = i;
				break;		// exit for loop
				}
			else if (StrCompare(midiListP[i].name, defaultName) == 0)
				{
				item = i;
				}
			}
		
		MemPtrUnlock(defaultName);
		
		// Set the list selection
		LstSetSelection (listP, item);

		// Init the sound trigger label
		// Create a new ptr to hold the label
		soundTriggerLabelP = MemPtrNew(soundTriggerLabelLen);
		// Check for mem failure
		ErrFatalDisplayIf(soundTriggerLabelP == NULL, "Out of memory");
		// Set the trigger label
		SetSoundLabel(formP, midiListP[item].name);

		// Unlock MIDI sound list
		MemPtrUnlock(midiListP);
		}
*/


static void SECT2 SetAlarmsFormClose(FormType * frmP)
{
	ControlType *ctlP;
	ListType *lstP;
	UInt8 value;
	
	GetObjectPtr(ctlP,SetAlarmsMorningAlarmCheckbox);
	g_prefs.enableMorningAlarm = CtlGetValue(ctlP);
	
	GetObjectPtr(ctlP,SetAlarmsEveningAlarmCheckbox);
	g_prefs.enableEveningAlarm = CtlGetValue(ctlP);
	
	GetObjectPtr(lstP,SetAlarmsMorningRepeatList);
	value=LstGetSelection(lstP);
	g_prefs.morningAlarmRepeat=value;
	
	GetObjectPtr(lstP,SetAlarmsEveningRepeatList);
	value=LstGetSelection(lstP);
	g_prefs.eveningAlarmRepeat=value;
	
	GetObjectPtr(ctlP,SetAlarmsDisableAlarmsCheckbox);
	g_prefs.disableAlarms = CtlGetValue(ctlP);
	
	//save prefences because I have to reload as local preferences in SetNFPAlarms
    PrefSetAppPreferences(
        appFileCreator, appPrefID, appPrefVersionNum, 
        &g_prefs, sizeof(NFPPreferenceType), true);
        
    FreeAll();
	
	SetNFPAlarms();		
}

Boolean SetAlarmsFormHandleEvent(EventType * eventP)
{
    Boolean handled = false;
    FormType * frmP;
    const Char *title;
    SndMidiListItemType* listP;
    UInt16 item;

    switch (eventP->eType) 
        {
        case frmOpenEvent:
            frmP = FrmGetActiveForm();
            SetAlarmsFormInit(frmP);
            FrmDrawForm(frmP);
            handled = true;
            break;
                         
        case frmUpdateEvent:
            // To do any custom drawing here, first call 
            // FrmDrawForm(), then do your drawing, and 
            // then set handled to true.
            break;
            
        case ctlSelectEvent:
	        switch (eventP->data.ctlSelect.controlID)
        	{
        		case SetAlarmsOKButton:
        			//manages data saving and deleting
        			//frmP = FrmGetActiveForm();
        			//SetAlarmsFormClose(frmP);
       
        			//returns to main form		
   			        GoToView();
        			handled = true;        		
        			break;
        			
        		case SetAlarmsMorningTimeButton:
        			//manages data saving and deleting
        			title = "";
        			SelectOneTime(&g_prefs.morningAlarmTime.hour,&g_prefs.morningAlarmTime.minute, title);
        			handled = true;        		
        			break;
        			
        		case SetAlarmsEveningTimeButton:
        			//manages data saving and deleting
        			title = "";
        			SelectOneTime(&g_prefs.eveningAlarmTime.hour,&g_prefs.eveningAlarmTime.minute, title);
        			handled = true;        		
        			break;	        			

        		default:
        			break;
			
			}
			break;
			
        case popSelectEvent:
	        switch (eventP->data.popSelect.listID)
        	{
				case SetAlarmsMorningSoundList:
					// Get new selected item
					item = eventP->data.popSelect.selection;
					// Get active form
					frmP = FrmGetActiveForm ();
					// Lock MIDI list
					listP = (SndMidiListItemType*)MemHandleLock(gMidiListH);
					// Save alarm sound unique rec ID
					g_prefs.morningAlarmSound = listP[item].uniqueRecID;
					// Erase control
					CtlEraseControl (eventP->data.popSelect.controlP);
					// Set new trigger label
					SetSoundLabelmor(frmP, listP[item].name);
					// Redraw control
					CtlDrawControl (eventP->data.popSelect.controlP);
					// Unlock MIDI list
					MemPtrUnlock(listP);
					// Play new alarm sound
					PlayAlarmSound (g_prefs.morningAlarmSound);
					// Mark event as handled
					handled = true;
					break;	
					
				case SetAlarmsEveningSoundList:
					// Get new selected item
					item = eventP->data.popSelect.selection;
					// Get active form
					frmP = FrmGetActiveForm ();
					// Lock MIDI list
					listP = (SndMidiListItemType*)MemHandleLock(gMidiListH);
					// Save alarm sound unique rec ID
					g_prefs.eveningAlarmSound = listP[item].uniqueRecID;
					// Erase control
					CtlEraseControl (eventP->data.popSelect.controlP);
					// Set new trigger label
					SetSoundLabeleve(frmP, listP[item].name);
					// Redraw control
					CtlDrawControl (eventP->data.popSelect.controlP);
					// Unlock MIDI list
					MemPtrUnlock(listP);
					// Play new alarm sound
					PlayAlarmSound (g_prefs.eveningAlarmSound);
					// Mark event as handled
					handled = true;
					break;						        			

        		default:
        			break;
			
			}
			break;			
			
		case frmCloseEvent:
   			//manages data saving and deleting
   			frmP = FrmGetActiveForm();
   			SetAlarmsFormClose(frmP);
		
			break;

        default:
            break;
        }
    
    return handled;
}

void HandleAlarmTrigger( SysAlarmTriggeredParamType * cmdPBP )
{
	LocalID dbID;
	UInt16 cardNo;
	UInt32 whichAlarm;
	DmOpenRef dbNFP;
	Boolean playSound;
	DateType today;
	UInt16 index;
	Boolean dataExists;
	Boolean beyondMaxIndex;
	Boolean isOldRecord = false;
	NFPDBRecord* item;
	DmSearchStateType searchInfo;
	NFPPreferenceType l_prefs;
	UInt16 prefsSize;
	UInt8 alarmRepeat;
	Boolean isDirty = false;
	UInt8 phase;
	Int16 cycleDay;
		
	whichAlarm = cmdPBP->ref;

	
	index = 0;
	
    // Read the saved preferences / saved-state information.
    prefsSize = sizeof(NFPPreferenceType);
    PrefGetAppPreferences(appFileCreator, appPrefID, &l_prefs, &prefsSize, true);
	
	DateSecondsToDate(TimGetSeconds(), &today);	//get today	
	
	// get the card number & dataBase ID for the app
	DmGetNextDatabaseByTypeCreator (true, &searchInfo, sysFileTApplication, appFileCreator, true, &cardNo, &dbID);
	
	//determine if morning or evening alarm
//	getAlarmReturn = AlmGetAlarm(cardNo, dbID, &whichAlarm);
	
	playSound = true;
	
	//Open Database
	dbNFP = DmOpenDatabaseByTypeCreator ( RECORD_DB_TYPE, appFileCreator, dmModeReadOnly );
	if(dbNFP != 0)
	{
	
		isOldRecord = FindRecordFromDateNonGlobal(today, &index, &dataExists, &beyondMaxIndex, dbNFP);
		if(isOldRecord)
		{
			item = DataQueryDataNonGlobal(index, dbNFP, &isDirty);

			//check to see if data has been entered for that alarm
			if(whichAlarm == 1)
			{
				if(item->basalTemp)
					playSound = false;
			//	if(l_prefs.morningAlarmSound == 0)
			//		playSound = false;
			}
			else if (whichAlarm == 2)
			{
				if(item->fluidSensation || item->fluidColor || item->cervix )
					playSound = false;
			//	if(l_prefs.eveningAlarmSound == 0)
			//		playSound = false;
			}
			DataReleaseDataNonGlobal (dbNFP, item, index, isDirty);
		}
	}
	
	if(dbNFP) DmCloseDatabase ( dbNFP );
	
	
	//if Alarms disabled after peak, then check Phase and turn off alarm if needed
	//only do this if prefsSize is correct, otherwise we may be dealing with an earlier version that does not have disableAlarms
	if(prefsSize == sizeof(NFPPreferenceType))
	{
		if(l_prefs.disableAlarms)
		{
			phase = DeterminePhase( today, index, isOldRecord, dataExists, beyondMaxIndex, &cycleDay, false, &l_prefs );
			if(phase == 5 || phase == 6) //ie infertile post-peak or pregnant
				playSound = false;
		}
	}
	
	
	//if not, sound the alarm, based on alarm preference
//	PlayAlarmSound(g_prefs.alarmSoundUniqueRecID);
//	PlayAlarmSound(g_prefs.morningAlarmSound);
//	PlayAlarmSound(g_prefs.eveningAlarmSound);
//	PlayAlarmSound(0); //default
	
	alarmRepeat = 1;
	if(whichAlarm == 1)
		alarmRepeat = l_prefs.morningAlarmRepeat;
	else if(whichAlarm == 2)
		alarmRepeat = l_prefs.eveningAlarmRepeat;
	

	if(playSound)
	{
		for(UInt8 i=1;i<=alarmRepeat;i++)
		{
			if(whichAlarm == 1)
				PlayAlarmSound(l_prefs.morningAlarmSound);
			else if(whichAlarm == 2)
				PlayAlarmSound(l_prefs.eveningAlarmSound);
			//SndPlaySystemSound (sndAlarm);
		}
	}
	
	//set next alarm
	SetNFPAlarms();

	return;
}


Boolean DisplayAlarm( SysAlarmTriggeredParamType * cmdPBP )
{
	LocalID dbID;
	UInt16 cardNo;
	DmOpenRef dbNFP;
	UInt32 whichAlarm;
    FormType * frmP;
    UInt16	button;
    Boolean startApp;
	DateType today;
	UInt32 now;
	UInt16 index;
	Boolean dataExists;
	Boolean beyondMaxIndex;
	Boolean isOldRecord = false;
	NFPDBRecord* item;
	DmSearchStateType searchInfo;
	Boolean showAlarm;
	Boolean isDirty = false;
	NFPPreferenceType l_prefs;
	NFPTempPreferenceType l_tempPrefs;
	UInt16 prefsSize;
	Char *pString;
	Char *pSecondString;
	Boolean match;
	Int16 compare;
	FieldType *fldP;
	UInt8 phase;
	Int16 cycleDay;
    
    // Read the saved preferences / saved-state information.
    prefsSize = sizeof(NFPTempPreferenceType);
    PrefGetAppPreferences(appFileCreator, appPrefID, &l_tempPrefs, &prefsSize, false);
    prefsSize = sizeof(NFPPreferenceType);
    PrefGetAppPreferences(appFileCreator, appPrefID, &l_prefs, &prefsSize, true);

    
	whichAlarm = cmdPBP->ref;
    
    DateSecondsToDate(TimGetSeconds(), &today);	//get today
    now = TimGetSeconds(); //get now
	
	// get the card number & dataBase ID for the app
	DmGetNextDatabaseByTypeCreator (true, &searchInfo, sysFileTApplication, appFileCreator, true, &cardNo, &dbID);
	
	//determine if morning or evening alarm
//	getAlarmReturn = AlmGetAlarm(cardNo, dbID, &whichAlarm);
	
	showAlarm = true;
	
	//see if an alarm has recently been displayed.  If so, don't display again
	if( (now - l_tempPrefs.lastDisplayedAlarm) < 30 )
		showAlarm  = false;
	
	//Open Database
	dbNFP = DmOpenDatabaseByTypeCreator ( RECORD_DB_TYPE, appFileCreator, dmModeReadOnly );
	if(dbNFP != 0)
	{
	
		isOldRecord = FindRecordFromDateNonGlobal(today, &index, &dataExists, &beyondMaxIndex, dbNFP);
		if(isOldRecord)
		{
			item = DataQueryDataNonGlobal(index, dbNFP, &isDirty);

			//check to see if data has been entered for that alarm
			if(whichAlarm == 1)
			{
				if(item->basalTemp)
					showAlarm = false;
			//	if(l_prefs.morningAlarmSound == 0)
			//		playSound = false;
			}
			else if (whichAlarm == 2)
			{
				if(item->fluidSensation || item->fluidColor || item->cervix )
					showAlarm = false;
			//	if(l_prefs.eveningAlarmSound == 0)
			//		playSound = false;
			}
			DataReleaseDataNonGlobal (dbNFP, item, index, isDirty);
		}
	}
	
	if(dbNFP) DmCloseDatabase ( dbNFP );

	//if Alarms disabled after peak, then check Phase and turn off alarm if needed
	if(prefsSize == sizeof(NFPPreferenceType))
	{
		if(l_prefs.disableAlarms)
		{
			phase = DeterminePhase( today, index, isOldRecord, dataExists, beyondMaxIndex, &cycleDay, false, &l_prefs );
			if(phase == 5 || phase == 6) //ie infertile post-peak or pregnant
				showAlarm = false;
		}
	}


	startApp = false;
	
    if(showAlarm)
    {
		//display AlarmForm
		frmP = FrmInitForm (AlarmForm);
		if(l_prefs.enablePassword)
		{
			FrmShowObject (frmP, FrmGetObjectIndex(frmP,AlarmEnterPasswordLabel));
			FrmShowObject (frmP, FrmGetObjectIndex(frmP,AlarmEnterPasswordField));		
			FrmSetFocus (frmP, FrmGetObjectIndex(frmP,AlarmEnterPasswordField));
		}
	    button = FrmDoDialog (frmP);                    
	  
		//if OK, display current date DataEntry
		//somehow create flag so DataEntry knows to quit app when done
		if(button == AlarmOKButton)
		{
			if(l_prefs.enablePassword)
			{
				fldP = (FieldType*) FrmGetObjectPtr(frmP,FrmGetObjectIndex(frmP,AlarmEnterPasswordField));
				pString = FldGetTextPtr(fldP);
				pSecondString = &l_prefs.password[0];
		
				if(pString!=NULL && pSecondString!=NULL)
				{
					compare = StrCompare(pString,pSecondString);
					if(compare==0)
						match = true;
					else
						match = false;
				}
				else if((pString==NULL || *pString==NULL) && (pSecondString==NULL || *pSecondString==NULL))
					match = true;
				else
					match = false;
					
				if(match) //if match, then startApp
					startApp = true;
				else
				{
					FrmAlert(PasswordMatchAlert);
					startApp = false;
				}
			
			}
			else
				startApp = true;

		}
		else //if Cancel, just close form
		{	
			startApp = false;
		}
		
		//record time that Alarm message was processed for comparison with future Alarm messages
		l_tempPrefs.lastDisplayedAlarm = TimGetSeconds();
	    PrefSetAppPreferences( appFileCreator, appPrefID, appPrefVersionNum, &l_tempPrefs, sizeof(NFPTempPreferenceType), false);
		
		FrmDeleteForm (frmP);
	}
	
	return startApp;
}


//ignores date and just compares time
Int16 TimeDiff(DateTimeType a, DateTimeType b)
{
	Int16 diff;
	
	diff = a.hour - b.hour;
	if(diff == 0)
		diff = a.minute - b.minute;
	
	return diff;
}

void SetNFPAlarms(void)
{
	UInt32 secAlarm;
	DateTimeType dttAlarm;
	UInt32 whichAlarm; //1 for morning, 2 for evening, 3 morning for next day, 4 evening alarm for next day
	UInt16 cardNo;
	Int16 morDiff;
	Int16 eveDiff;
	Int16 biggerDiff;
	LocalID dbID;
	DmSearchStateType searchInfo;
	DateTimeType now;
    UInt16 prefsSize;
    Int16 prefVersion;
    NFPPreferenceType l_prefs;
    Err err;

//    UInt32 alarmReturn;
//   	UInt32 testwhich;
    
    // Read the saved preferences / saved-state information.
    prefsSize = sizeof(NFPPreferenceType);
    prefVersion = PrefGetAppPreferences(appFileCreator, appPrefID, &l_prefs, &prefsSize, true);
	
	//if this is the first time the program is being run, the preferences will not exist, so exit
	if(prefVersion == noPreferenceFound)
		return;
	
	// get the card number & dataBase ID for the app
	// by database, we mean the program itself which should always exist
//	cardNo = 0; //assume 0
//	dbID = DmFindDatabase(0,"NFP");
	err=DmGetNextDatabaseByTypeCreator (true, &searchInfo, sysFileTApplication, appFileCreator, true, &cardNo, &dbID);
	ErrFatalDisplayIf(err,"Could not get dbID for application inorder to set Alarm");
	

	if(l_prefs.enableMorningAlarm || l_prefs.enableEveningAlarm)
	{
		//get current time
		TimSecondsToDateTime(TimGetSeconds(),&now);
	
		morDiff = TimeDiff(now,l_prefs.morningAlarmTime);
		eveDiff = TimeDiff(now,l_prefs.eveningAlarmTime);
		biggerDiff = TimeDiff(l_prefs.eveningAlarmTime,l_prefs.morningAlarmTime);
	
		//normal situation, morning Alarm is before evening Alarm
		if( biggerDiff >= 0 )
		{
			if(morDiff < 0 && eveDiff < 0)
				whichAlarm = 1;
			else if(morDiff >= 0 && eveDiff < 0)
				whichAlarm = 2;
			else //if(morDiff >= 0 && eveDiff >= 0) //to ensure whichAlarm is initialized remove if clause
				whichAlarm = 3;

			//sort out enabled checkboxes
			//we know at least one is enabled
			if(whichAlarm == 1 && !l_prefs.enableMorningAlarm)
				whichAlarm = 2;
			else if(whichAlarm == 2 && !l_prefs.enableEveningAlarm)
				whichAlarm = 3;
			else if(whichAlarm == 3 && !l_prefs.enableMorningAlarm)
				whichAlarm = 4;
		}
		//evening alarm is first
		else if( biggerDiff < 0 )
		{
			if(morDiff < 0 && eveDiff < 0)
				whichAlarm = 2;
			else if(morDiff < 0 && eveDiff >= 0)
				whichAlarm = 1;
			else //if(morDiff >= 0 && eveDiff >= 0) //to ensure whichAlarm is initialized remove if clause
				whichAlarm = 4;
				
			//sort out enabled checkboxes
			//we know at least one is enabled
			if(whichAlarm == 1 && !l_prefs.enableMorningAlarm)
				whichAlarm = 4;
			else if(whichAlarm == 2 && !l_prefs.enableEveningAlarm)
				whichAlarm = 1;
			else if(whichAlarm == 4 && !l_prefs.enableEveningAlarm)
				whichAlarm = 3;		
		}
	
		

	
		dttAlarm = now;
		dttAlarm.second = 0;	
		//set appropriate Alarm
		if(whichAlarm == 1)
		{
			dttAlarm.minute = l_prefs.morningAlarmTime.minute;
			dttAlarm.hour = l_prefs.morningAlarmTime.hour;
			secAlarm = TimDateTimeToSeconds(&dttAlarm);
			err = AlmSetAlarm(cardNo, dbID, 1, secAlarm, NULL );
		}
		else if(whichAlarm == 2)
		{
			dttAlarm.minute = l_prefs.eveningAlarmTime.minute;
			dttAlarm.hour = l_prefs.eveningAlarmTime.hour;
			secAlarm = TimDateTimeToSeconds(&dttAlarm);
			err = AlmSetAlarm(cardNo, dbID, 2, secAlarm, NULL );
		}
		else if(whichAlarm == 3)
		{
			dttAlarm.minute = l_prefs.morningAlarmTime.minute;
			dttAlarm.hour = l_prefs.morningAlarmTime.hour;
			TimAdjust( &dttAlarm, 86400 );//increment one day
			secAlarm = TimDateTimeToSeconds(&dttAlarm);
			//whichAlarm = 1
			err = AlmSetAlarm(cardNo, dbID, 1, secAlarm, NULL );
		}
		else if(whichAlarm == 4)
		{
			dttAlarm.minute = l_prefs.eveningAlarmTime.minute;
			dttAlarm.hour = l_prefs.eveningAlarmTime.hour;
			TimAdjust( &dttAlarm, 86400 );//increment one day
			secAlarm = TimDateTimeToSeconds(&dttAlarm);
			err = AlmSetAlarm(cardNo, dbID, 2, secAlarm, NULL );
		}
	}
	else
	{
		//clear out any alarms
		err = AlmSetAlarm(cardNo, dbID, NULL, 0, NULL );
	}
	
//	alarmReturn = AlmGetAlarm(cardNo, dbID, &testwhich);
	
	return;
}